@@ -15,7 +15,6 @@
 
 import re
 import heapq
-import pyee
 from adapt.entity_tagger import EntityTagger
 from adapt.parser import Parser
 from adapt.tools.text.tokenizer import EnglishTokenizer
@@ -24,7 +23,7 @@ from adapt.tools.text.trie import Trie
 __author__ = 'seanfitz'
 
 
-class IntentDeterminationEngine(pyee.EventEmitter):
+class IntentDeterminationEngine(object):
     """
     IntentDeterminationEngine
 
@@ -34,7 +33,7 @@ class IntentDeterminationEngine(pyee.EventEmitter):
     weighted based on the percentage of the utterance (per character) that the entity match represents.
 
     This system makes heavy use of generators to enable greedy algorithms to short circuit large portions of
-    computation.
+    computation, however making use of context or regular expressions prevents these optimizations.
     """
     def __init__(self, tokenizer=None, trie=None):
         """
@@ -45,12 +44,10 @@ class IntentDeterminationEngine(pyee.EventEmitter):
                 example EnglishTokenizer()
             trie(Trie): tree of matches to Entites
         """
-        pyee.EventEmitter.__init__(self)
         self.tokenizer = tokenizer or EnglishTokenizer()
         self.trie = trie or Trie()
         self.regular_expressions_entities = []
         self._regex_strings = set()
-        self.tagger = EntityTagger(self.trie, self.tokenizer, self.regular_expressions_entities)
         self.intent_parsers = []
 
     def __best_intent(self, parse_result, context=[]):
@@ -102,6 +99,11 @@ class IntentDeterminationEngine(pyee.EventEmitter):
         result_context = [c for c in context if c['key'] not in tags_keys]
         return result_context
 
+    @property
+    def tagger(self):
+        return EntityTagger(self.trie, self.tokenizer,
+                     self.regular_expressions_entities)
+
     def determine_intent(self, utterance, num_results=1, include_tags=False, context_manager=None):
         """
         Given an utterance, provide a valid intent.
@@ -116,23 +118,36 @@ class IntentDeterminationEngine(pyee.EventEmitter):
         Returns: A generator that yields dictionaries.
         """
         parser = Parser(self.tokenizer, self.tagger)
-        parser.on('tagged_entities',
-                  (lambda result:
-                   self.emit("tagged_entities", result)))
 
         context = []
         if context_manager:
             context = context_manager.get_context()
 
-        for result in parser.parse(utterance, N=num_results, context=context):
-            self.emit("parse_result", result)
-            # create a context without entities used in result
-            remaining_context = self.__get_unused_context(result, context)
-            best_intent, tags = self.__best_intent(result, remaining_context)
-            if best_intent and best_intent.get('confidence', 0.0) > 0:
-                if include_tags:
-                    best_intent['__tags__'] = tags
-                yield best_intent
+        # Adapt consumers assume that results are sorted by confidence. parser
+        # will yield results sorted by utterance coverage, but regex
+        # and context entities will have different weights, and
+        # can influence final sorting.
+        requires_final_sort = self.regular_expressions_entities or context
+
+        def generate_intents():
+            for result in parser.parse(utterance, N=num_results, context=context):
+                # create a context without entities used in result
+                remaining_context = self.__get_unused_context(result, context)
+                best_intent, tags = self.__best_intent(result, remaining_context)
+                if best_intent and best_intent.get('confidence', 0.0) > 0:
+                    if include_tags:
+                        best_intent['__tags__'] = tags
+                    yield best_intent
+
+        if requires_final_sort:
+            sorted_iterable = sorted([
+                i for i in generate_intents()
+            ], key=lambda x: -x.get('confidence', 0.0))
+        else:
+            sorted_iterable = generate_intents()
+
+        for intent in sorted_iterable:
+            yield intent
 
     def register_entity(self, entity_value, entity_type, alias_of=None):
         """
